@@ -3,16 +3,19 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2007, 2014, 2016 Lukáš Lalinský
-# Copyright (C) 2014, 2019-2020 Philipp Wolfer
-# Copyright (C) 2014-2016, 2018-2020 Laurent Monin
+# Copyright (C) 2014, 2019-2021 Philipp Wolfer
+# Copyright (C) 2014-2016, 2018-2021 Laurent Monin
 # Copyright (C) 2015 Ohm Patel
 # Copyright (C) 2016 Rahul Raturi
 # Copyright (C) 2016 Wieland Hoffmann
 # Copyright (C) 2016-2017 Frederik “Freso” S. Olesen
 # Copyright (C) 2017 Antonio Larrosa
 # Copyright (C) 2017 Sambhav Kothari
-# Copyright (C) 2018 Bob Swift
 # Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2018, 2021 Bob Swift
+# Copyright (C) 2020 RomFouq
+# Copyright (C) 2021 Gabriel Ferreira
+# Copyright (C) 2021 Vladislav Karbovskii
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,33 +32,28 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
+import builtins
 from collections import OrderedDict
-import os
 
-from PyQt5.QtCore import QStandardPaths
-from picard import (
-    PICARD_APP_NAME,
-    PICARD_ORG_NAME,
-)
+from picard import PICARD_VERSION
+from picard.const.attributes import MB_ATTRIBUTES
+from picard.const import appdirs
 
 # Install gettext "noop" function in case const.py gets imported directly.
-import builtins
 builtins.__dict__['N_'] = lambda a: a
 
 
 # Config directory
-_appconfiglocation = QStandardPaths.writableLocation(QStandardPaths.AppConfigLocation)
-USER_DIR = os.path.normpath(os.path.join(_appconfiglocation, PICARD_ORG_NAME, PICARD_APP_NAME))
-USER_PLUGIN_DIR = os.path.normpath(os.path.join(USER_DIR, "plugins"))
+USER_DIR = appdirs.config_folder()
+USER_PLUGIN_DIR = appdirs.plugin_folder()
 
 # Network Cache default settings
-CACHE_DIR = os.path.normpath(QStandardPaths.writableLocation(QStandardPaths.CacheLocation))
 CACHE_SIZE_IN_BYTES = 100*1000*1000
 
 # AcoustID client API key
 ACOUSTID_KEY = 'v8pQ6oyB'
 ACOUSTID_HOST = 'api.acoustid.org'
-ACOUSTID_PORT = 80
+ACOUSTID_PORT = 443
 FPCALC_NAMES = ['fpcalc', 'pyfpcalc']
 
 # MB OAuth client credentials
@@ -66,20 +64,30 @@ MUSICBRAINZ_OAUTH_CLIENT_SECRET = 'xIsvXbIuntaLuRRhzuazOA'
 CAA_HOST = "coverartarchive.org"
 CAA_PORT = 443
 
+# Prepare documentation URLs
+if PICARD_VERSION.identifier == 'final':
+    DOCS_VERSION = "v{}.{}/".format(PICARD_VERSION.major, PICARD_VERSION.minor)
+else:
+    DOCS_VERSION = ""  # points to latest version
+DOCS_LANGUAGE = 'en'
+DOCS_BASE_URL = "https://picard-docs.musicbrainz.org/" + DOCS_VERSION + DOCS_LANGUAGE
+
 # URLs
 PICARD_URLS = {
-    'documentation':    "https://picard.musicbrainz.org/docs/",
-    'troubleshooting':  "https://picard.musicbrainz.org/docs/troubleshooting/",
-    'home':             "https://picard.musicbrainz.org/",
-    'doc_options':      "https://picard.musicbrainz.org/docs/options/",
-    'doc_scripting':    "https://picard.musicbrainz.org/docs/scripting",
-    'plugins':          "https://picard.musicbrainz.org/plugins/",
-    'forum':            "https://community.metabrainz.org/c/picard",
-    'donate':           "https://metabrainz.org/donate",
-    'chromaprint':      "https://acoustid.org/chromaprint#download",
-    'acoustid_apikey':  "https://acoustid.org/api-key",
-    'doc_cover_art_types': "https://musicbrainz.org/doc/Cover_Art/Types",
-    'acoustid_track': "https://acoustid.org/track/",
+    'home':                    "https://picard.musicbrainz.org/",
+    'documentation':           DOCS_BASE_URL + '/',
+    'troubleshooting':         DOCS_BASE_URL + '/troubleshooting/troubleshooting.html',
+    'doc_options':             DOCS_BASE_URL + '/config/configuration.html',
+    'doc_scripting':           DOCS_BASE_URL + '/extending/scripting.html',
+    'doc_tags_from_filenames': DOCS_BASE_URL + '/usage/tags_from_file_names.html',
+    'doc_naming_script_edit':  DOCS_BASE_URL + '/config/options_filerenaming_editor.html',
+    'doc_cover_art_types':     "https://musicbrainz.org/doc/Cover_Art/Types",
+    'plugins':                 "https://picard.musicbrainz.org/plugins/",
+    'forum':                   "https://community.metabrainz.org/c/picard",
+    'donate':                  "https://metabrainz.org/donate",
+    'chromaprint':             "https://acoustid.org/chromaprint#download",
+    'acoustid_apikey':         "https://acoustid.org/api-key",
+    'acoustid_track':          "https://acoustid.org/track/",
 }
 
 # Various Artists MBID
@@ -90,7 +98,6 @@ SILENCE_TRACK_TITLE = '[silence]'
 DATA_TRACK_TITLE = '[data track]'
 
 # Release formats
-from picard.const.attributes import MB_ATTRIBUTES
 RELEASE_FORMATS = {}
 RELEASE_PRIMARY_GROUPS = {}
 RELEASE_SECONDARY_GROUPS = {}
@@ -105,14 +112,17 @@ for k, v in MB_ATTRIBUTES.items():
     elif k.startswith('DB:release_status/name:'):
         RELEASE_STATUS[v] = v
 
+# List of available charsets
+from picard.const.scripts import SCRIPTS  # noqa: F401,E402 # pylint: disable=unused-import
+
 # Release countries
-from picard.const.countries import RELEASE_COUNTRIES  # noqa: F401 # pylint: disable=unused-import
+from picard.const.countries import RELEASE_COUNTRIES  # noqa: F401,E402 # pylint: disable=unused-import
 
 # List of available user interface languages
-from picard.const.languages import UI_LANGUAGES  # noqa: F401 # pylint: disable=unused-import
+from picard.const.languages import UI_LANGUAGES  # noqa: F401,E402 # pylint: disable=unused-import
 
 # List of alias locales
-from picard.const.locales import ALIAS_LOCALES  # noqa: F401 # pylint: disable=unused-import
+from picard.const.locales import ALIAS_LOCALES  # noqa: F401,E402 # pylint: disable=unused-import
 
 # List of official musicbrainz servers - must support SSL for mblogin requests (such as collections).
 MUSICBRAINZ_SERVERS = [
@@ -132,7 +142,7 @@ PLUGINS_API = {
 }
 
 # Default query limit
-QUERY_LIMIT = 25
+QUERY_LIMIT = 50
 
 # Maximum number of covers to draw in a stack in CoverArtThumbnail
 MAX_COVERS_TO_STACK = 4
@@ -164,11 +174,17 @@ PROGRAM_UPDATE_LEVELS = OrderedDict(
 
 DEFAULT_FILE_NAMING_FORMAT = "$if2(%albumartist%,%artist%)/\n" \
     "$if(%albumartist%,%album%/,)\n" \
-    "$if($gt(%totaldiscs%,1),%discnumber%-,)" \
+    "$if($gt(%totaldiscs%,1),$if($gt(%totaldiscs%,9),$num(%discnumber%,2),%discnumber%)-,)" \
     "$if($and(%albumartist%,%tracknumber%),$num(%tracknumber%,2) ,)" \
     "$if(%_multiartist%,%artist% - ,)" \
     "%title%"
 
 
-DEFAULT_NUMBERED_SCRIPT_NAME = N_("My script %d")
 DEFAULT_SCRIPT_NAME = N_("My script")
+DEFAULT_COVER_IMAGE_FILENAME = "cover"
+DEFAULT_PROFILE_NAME = N_("My profile")
+DEFAULT_COPY_TEXT = N_("(copy)")
+DEFAULT_NUMBERED_TITLE_FORMAT = N_("{title} ({count})")
+DEFAULT_NAMING_PRESET_ID = "Preset 1"
+
+SCRIPT_LANGUAGE_VERSION = '1.1'
