@@ -4,10 +4,10 @@
 #
 # Copyright (C) 2007 Oliver Charles
 # Copyright (C) 2007, 2010-2011 Lukáš Lalinský
-# Copyright (C) 2007-2011, 2015, 2018-2019 Philipp Wolfer
+# Copyright (C) 2007-2011, 2015, 2018-2022 Philipp Wolfer
 # Copyright (C) 2011 Michael Wiencek
 # Copyright (C) 2011-2012 Wieland Hoffmann
-# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2015, 2018-2021 Laurent Monin
 # Copyright (C) 2015-2016 Rahul Raturi
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Frederik “Freso” S. Olesen
@@ -45,9 +45,12 @@ from PyQt5.QtNetwork import (
     QNetworkRequest,
 )
 
-from picard import (
-    config,
-    log,
+from picard import log
+from picard.config import (
+    BoolOption,
+    IntOption,
+    ListOption,
+    get_config,
 )
 from picard.const import (
     CAA_HOST,
@@ -65,7 +68,7 @@ from picard.coverart.utils import (
     CAA_TYPES,
     translate_caa_type,
 )
-from picard.util import webbrowser2
+from picard.webservice import ratecontrol
 
 from picard.ui import PicardDialog
 from picard.ui.ui_provider_options_caa import Ui_CaaOptions
@@ -87,7 +90,10 @@ _CAA_THUMBNAIL_SIZE_ALIASES = {
 _CAA_IMAGE_SIZE_DEFAULT = 500
 
 _CAA_IMAGE_TYPE_DEFAULT_INCLUDE = ['front']
-_CAA_IMAGE_TYPE_DEFAULT_EXCLUDE = ['raw/unedited', 'watermark']
+_CAA_IMAGE_TYPE_DEFAULT_EXCLUDE = ['matrix/runout', 'raw/unedited', 'watermark']
+
+ratecontrol.set_minimum_delay((CAA_HOST, CAA_PORT), 0)
+ratecontrol.set_minimum_delay(('archive.org', 443), 0)
 
 
 def caa_url_fallback_list(desired_size, thumbnails):
@@ -148,7 +154,7 @@ class ArrowsColumn(QtWidgets.QWidget):
         self.selection_list = selection_list
         self.ignore_list = ignore_list
         self.callback = callback
-        spacer_item = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
+        spacer_item = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
         arrows_layout = QtWidgets.QVBoxLayout()
         arrows_layout.addItem(QtWidgets.QSpacerItem(spacer_item))
         self.button_add = ArrowButton('go-next' if reverse else 'go-previous', self.move_from_ignore)
@@ -188,9 +194,9 @@ class ListBox(QtWidgets.QListWidget):
     def __init__(self, parent=None):
         super().__init__(parent=parent)
         self.setMinimumSize(QtCore.QSize(self.LISTBOX_WIDTH, self.LISTBOX_HEIGHT))
-        self.setSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
+        self.setSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Expanding)
         self.setSortingEnabled(True)
-        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
 
     def move_item(self, item, target_list):
         """Move the specified item to another listbox."""
@@ -211,7 +217,7 @@ class ListBox(QtWidgets.QListWidget):
         if callback:
             callback()
 
-    def all_items_data(self, role=QtCore.Qt.UserRole):
+    def all_items_data(self, role=QtCore.Qt.ItemDataRole.UserRole):
         for index in range(self.count()):
             yield self.item(index).data(role)
 
@@ -225,6 +231,8 @@ class CAATypesSelectorDialog(PicardDialog):
         types_exclude {[string]} -- List of CAA image types to exclude (default: {None})
     """
 
+    help_url = 'doc_cover_art_types'
+
     def __init__(self, parent=None, types_include=None, types_exclude=None):
         super().__init__(parent)
         if types_include is None:
@@ -233,9 +241,9 @@ class CAATypesSelectorDialog(PicardDialog):
             types_exclude = []
 
         self.setWindowTitle(_("Cover art types"))
-        self.setWindowModality(QtCore.Qt.WindowModal)
+        self.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
         self.layout = QtWidgets.QVBoxLayout(self)
-        self.layout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
+        self.layout.setSizeConstraint(QtWidgets.QLayout.SizeConstraint.SetFixedSize)
 
         # Create list boxes for dialog
         self.list_include = ListBox()
@@ -254,7 +262,7 @@ class CAATypesSelectorDialog(PicardDialog):
         instructions = QtWidgets.QLabel()
         instructions.setText(_("Please select the contents of the image type 'Include' and 'Exclude' lists."))
         instructions.setWordWrap(True)
-        instructions.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
+        instructions.setSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
         self.layout.addWidget(instructions)
 
         self.arrows_include = ArrowsColumn(
@@ -303,17 +311,17 @@ class CAATypesSelectorDialog(PicardDialog):
             "use a CAA image.\n")
         )
         instructions.setWordWrap(True)
-        instructions.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
+        instructions.setSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
         self.layout.addWidget(instructions)
 
         self.buttonbox = QtWidgets.QDialogButtonBox(self)
-        self.buttonbox.setOrientation(QtCore.Qt.Horizontal)
+        self.buttonbox.setOrientation(QtCore.Qt.Orientation.Horizontal)
         self.buttonbox.addButton(
-            StandardButton(StandardButton.OK), QtWidgets.QDialogButtonBox.AcceptRole)
+            StandardButton(StandardButton.OK), QtWidgets.QDialogButtonBox.ButtonRole.AcceptRole)
         self.buttonbox.addButton(StandardButton(StandardButton.CANCEL),
-                                 QtWidgets.QDialogButtonBox.RejectRole)
+                                 QtWidgets.QDialogButtonBox.ButtonRole.RejectRole)
         self.buttonbox.addButton(
-            StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.HelpRole)
+            StandardButton(StandardButton.HELP), QtWidgets.QDialogButtonBox.ButtonRole.HelpRole)
 
         extrabuttons = [
             (N_("I&nclude all"), self.move_all_to_include_list),
@@ -323,14 +331,14 @@ class CAATypesSelectorDialog(PicardDialog):
         ]
         for label, callback in extrabuttons:
             button = QtWidgets.QPushButton(_(label))
-            self.buttonbox.addButton(button, QtWidgets.QDialogButtonBox.ActionRole)
+            self.buttonbox.addButton(button, QtWidgets.QDialogButtonBox.ButtonRole.ActionRole)
             button.clicked.connect(callback)
 
         self.layout.addWidget(self.buttonbox)
 
         self.buttonbox.accepted.connect(self.accept)
         self.buttonbox.rejected.connect(self.reject)
-        self.buttonbox.helpRequested.connect(self.help)
+        self.buttonbox.helpRequested.connect(self.show_help)
 
         self.set_buttons_enabled_state()
 
@@ -367,7 +375,7 @@ class CAATypesSelectorDialog(PicardDialog):
             name = caa_type['name']
             title = translate_caa_type(name)
             item = QtWidgets.QListWidgetItem(title)
-            item.setData(QtCore.Qt.UserRole, name)
+            item.setData(QtCore.Qt.ItemDataRole.UserRole, name)
             if name in includes:
                 self.list_include.addItem(item)
             elif name in excludes:
@@ -375,9 +383,6 @@ class CAATypesSelectorDialog(PicardDialog):
             else:
                 self.list_ignore.addItem(item)
 
-    def help(self):
-        webbrowser2.goto('doc_cover_art_types')
-
     def get_selected_types_include(self):
         return list(self.list_include.all_items_data()) or ['front']
 
@@ -422,7 +427,7 @@ class CAATypesSelectorDialog(PicardDialog):
             types_exclude = []
         dialog = CAATypesSelectorDialog(parent, types_include, types_exclude)
         result = dialog.exec_()
-        return (dialog.get_selected_types_include(), dialog.get_selected_types_exclude(), result == QtWidgets.QDialog.Accepted)
+        return (dialog.get_selected_types_include(), dialog.get_selected_types_exclude(), result == QtWidgets.QDialog.DialogCode.Accepted)
 
 
 class ProviderOptionsCaa(ProviderOptions):
@@ -430,16 +435,15 @@ class ProviderOptionsCaa(ProviderOptions):
         Options for Cover Art Archive cover art provider
     """
 
+    TITLE = N_("Cover Art Archive")
     HELP_URL = '/config/options_cover_art_archive.html'
 
     options = [
-        config.BoolOption("setting", "caa_save_single_front_image", False),
-        config.BoolOption("setting", "caa_approved_only", False),
-        config.BoolOption("setting", "caa_image_type_as_filename", False),
-        config.IntOption("setting", "caa_image_size", _CAA_IMAGE_SIZE_DEFAULT),
-        config.ListOption("setting", "caa_image_types", _CAA_IMAGE_TYPE_DEFAULT_INCLUDE),
-        config.BoolOption("setting", "caa_restrict_image_types", True),
-        config.ListOption("setting", "caa_image_types_to_omit", _CAA_IMAGE_TYPE_DEFAULT_EXCLUDE),
+        BoolOption("setting", "caa_approved_only", False),
+        IntOption("setting", "caa_image_size", _CAA_IMAGE_SIZE_DEFAULT),
+        ListOption("setting", "caa_image_types", _CAA_IMAGE_TYPE_DEFAULT_INCLUDE),
+        BoolOption("setting", "caa_restrict_image_types", True),
+        ListOption("setting", "caa_image_types_to_omit", _CAA_IMAGE_TYPE_DEFAULT_EXCLUDE),
     ]
 
     _options_ui = Ui_CaaOptions
@@ -459,15 +463,14 @@ class ProviderOptionsCaa(ProviderOptions):
         for item_id, item in _CAA_THUMBNAIL_SIZE_MAP.items():
             self.ui.cb_image_size.addItem(_(item.label), userData=item_id)
 
+        config = get_config()
         size = config.setting["caa_image_size"]
         index = self.ui.cb_image_size.findData(size)
         if index < 0:
             index = self.ui.cb_image_size.findData(_CAA_IMAGE_SIZE_DEFAULT)
         self.ui.cb_image_size.setCurrentIndex(index)
 
-        self.ui.cb_save_single_front_image.setChecked(config.setting["caa_save_single_front_image"])
         self.ui.cb_approved_only.setChecked(config.setting["caa_approved_only"])
-        self.ui.cb_type_as_filename.setChecked(config.setting["caa_image_type_as_filename"])
         self.ui.restrict_images_types.setChecked(
             config.setting["caa_restrict_image_types"])
         self.caa_image_types = config.setting["caa_image_types"]
@@ -475,14 +478,11 @@ class ProviderOptionsCaa(ProviderOptions):
         self.update_caa_types()
 
     def save(self):
+        config = get_config()
         size = self.ui.cb_image_size.currentData()
         config.setting["caa_image_size"] = size
-        config.setting["caa_save_single_front_image"] = \
-            self.ui.cb_save_single_front_image.isChecked()
         config.setting["caa_approved_only"] = \
             self.ui.cb_approved_only.isChecked()
-        config.setting["caa_image_type_as_filename"] = \
-            self.ui.cb_type_as_filename.isChecked()
         config.setting["caa_restrict_image_types"] = \
             self.ui.restrict_images_types.isChecked()
         config.setting["caa_image_types"] = self.caa_image_types
@@ -505,7 +505,7 @@ class CoverArtProviderCaa(CoverArtProvider):
     """Get cover art from Cover Art Archive using release mbid"""
 
     NAME = "Cover Art Archive"
-    TITLE = N_('Cover Art Archive')
+    TITLE = N_('Cover Art Archive: Release')
     OPTIONS = ProviderOptionsCaa
 
     ignore_json_not_found_error = False
@@ -514,6 +514,7 @@ class CoverArtProviderCaa(CoverArtProvider):
 
     def __init__(self, coverart):
         super().__init__(coverart)
+        config = get_config()
         self.caa_types = list(map(str.lower, config.setting["caa_image_types"]))
         self.caa_types_to_omit = list(map(str.lower, config.setting["caa_image_types_to_omit"]))
         self.len_caa_types = len(self.caa_types)
@@ -587,7 +588,7 @@ class CoverArtProviderCaa(CoverArtProvider):
             self._caa_json_downloaded,
             priority=True,
             important=False,
-            cacheloadcontrol=QNetworkRequest.PreferNetwork
+            cacheloadcontrol=QNetworkRequest.CacheLoadControl.PreferNetwork
         )
         self.album._requests += 1
         # we will call next_in_queue() after json parsing
@@ -597,12 +598,13 @@ class CoverArtProviderCaa(CoverArtProvider):
         """Parse CAA JSON file and queue CAA cover art images for download"""
         self.album._requests -= 1
         if error:
-            if not (error == QNetworkReply.ContentNotFoundError and self.ignore_json_not_found_error):
+            if not (error == QNetworkReply.NetworkError.ContentNotFoundError and self.ignore_json_not_found_error):
                 self.error('CAA JSON error: %s' % (http.errorString()))
         else:
             if self.restrict_types:
                 log.debug('CAA types: included: %s, excluded: %s' % (self.caa_types, self.caa_types_to_omit,))
             try:
+                config = get_config()
                 for image in data["images"]:
                     if config.setting["caa_approved_only"] and not image["approved"]:
                         continue
@@ -658,7 +660,7 @@ class CoverArtProviderCaa(CoverArtProvider):
                             # PDFs cannot be saved to tags (as 2014/05/29)
                             coverartimage.can_be_saved_to_tags = False
                         self.queue_put(coverartimage)
-                        if config.setting["caa_save_single_front_image"] and \
+                        if config.setting["save_only_one_front_image"] and \
                                 config.setting["save_images_to_files"] and \
                                 image["front"]:
                             break
