@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2007 Lukáš Lalinský
 # Copyright (C) 2014 Shadab Zafar
-# Copyright (C) 2015-2019 Laurent Monin
+# Copyright (C) 2015-2021 Laurent Monin
 # Copyright (C) 2019 Wieland Hoffmann
 # Copyright (C) 2019-2020 Philipp Wolfer
 #
@@ -208,11 +208,11 @@ class PluginManager(QtCore.QObject):
             return
         if plugindir == self.plugins_directory:
             # .update trick is only for plugins installed through the Picard UI
-            # and only for plugins in plugins_directory (USER_PLUGIN_DIR by default)
+            # and only for plugins in plugins_directory (USER_PLUGIN_DIR by default)
             self.handle_plugin_updates()
         # now load found plugins
         names = set()
-        for path in [os.path.join(plugindir, file) for file in os.listdir(plugindir)]:
+        for path in (os.path.join(plugindir, file) for file in os.listdir(plugindir)):
             name = _plugin_name_from_path(path)
             if name:
                 names.add(name)
@@ -301,9 +301,10 @@ class PluginManager(QtCore.QObject):
         dirpath = os.path.join(self.plugins_directory, plugin_name)
         if not os.path.isdir(dirpath):
             dirpath = None
+        filenames = {plugin_name + ext for ext in fileexts}
         filepaths = [os.path.join(self.plugins_directory, f)
                      for f in os.listdir(self.plugins_directory)
-                     if f in [plugin_name + ext for ext in fileexts]
+                     if f in filenames
                      ]
         return (dirpath, filepaths)
 
@@ -395,7 +396,7 @@ class PluginManager(QtCore.QObject):
                     self._install_plugin_file(path, update=update)
                 elif os.path.isdir(path):
                     self._install_plugin_dir(plugin_name, path, update=update)
-            except (OSError, IOError) as why:
+            except OSError as why:
                 log.error("Unable to copy plugin '%s' to %r: %s" % (plugin_name, self.plugins_directory, why))
                 return
 
