@@ -6,7 +6,7 @@
 # Copyright (C) 2006-2009, 2011-2014, 2017 Lukáš Lalinský
 # Copyright (C) 2008 Gary van der Merwe
 # Copyright (C) 2008 amckinle
-# Copyright (C) 2008-2010, 2014-2015, 2018-2021 Philipp Wolfer
+# Copyright (C) 2008-2010, 2014-2015, 2018-2022 Philipp Wolfer
 # Copyright (C) 2009 Carlin Mangar
 # Copyright (C) 2010 Andrew Barnert
 # Copyright (C) 2011-2014 Michael Wiencek
@@ -16,7 +16,7 @@
 # Copyright (C) 2013 Ionuț Ciocîrlan
 # Copyright (C) 2013 brainz34
 # Copyright (C) 2013-2014, 2017 Sophist-UK
-# Copyright (C) 2013-2015, 2017-2021 Laurent Monin
+# Copyright (C) 2013-2015, 2017-2022 Laurent Monin
 # Copyright (C) 2016 Rahul Raturi
 # Copyright (C) 2016 Simon Legner
 # Copyright (C) 2016 Suhas
@@ -44,9 +44,7 @@
 
 
 import argparse
-from concurrent.futures import ThreadPoolExecutor
 from functools import partial
-import itertools
 import logging
 import os.path
 import platform
@@ -70,11 +68,6 @@ from picard import (
     acoustid,
     log,
 )
-from picard.acousticbrainz import (
-    ABExtractor,
-    ab_extractor_callback,
-    ab_feature_extraction,
-)
 from picard.acoustid.manager import AcoustIDManager
 from picard.album import (
     Album,
@@ -101,10 +94,15 @@ from picard.const import (
 from picard.const.sys import (
     IS_FROZEN,
     IS_HAIKU,
+    IS_MACOS,
     IS_WIN,
 )
 from picard.dataobj import DataObject
-from picard.disc import Disc
+from picard.disc import (
+    Disc,
+    eaclog,
+    whipperlog,
+)
 from picard.file import File
 from picard.formats import open_ as open_file
 from picard.i18n import setup_gettext
@@ -122,7 +120,9 @@ from picard.util import (
     iter_files_from_objects,
     mbid_validate,
     normpath,
+    periodictouch,
     process_events_iter,
+    system_supports_long_paths,
     thread,
     versions,
     webbrowser2,
@@ -184,7 +184,7 @@ class Tagger(QtWidgets.QApplication):
     _debug = False
     _no_restore = False
 
-    def __init__(self, picard_args, unparsed_args, localedir, autoupdate):
+    def __init__(self, picard_args, localedir, autoupdate):
 
         super().__init__(sys.argv)
         self.__class__.__instance = self
@@ -202,18 +202,21 @@ class Tagger(QtWidgets.QApplication):
         if picard_args.debug or "PICARD_DEBUG" in os.environ:
             self.set_log_level(logging.DEBUG)
 
-        # Default thread pool
-        self.thread_pool = ThreadPoolExecutor()
+        # FIXME: Figure out what's wrong with QThreadPool.globalInstance().
+        # It's a valid reference, but its start() method doesn't work.
+        self.thread_pool = QtCore.QThreadPool(self)
 
         # Provide a separate thread pool for operations that should not be
         # delayed by longer background processing tasks, e.g. because the user
         # expects instant feedback instead of waiting for a long list of
         # operations to finish.
-        self.priority_thread_pool = ThreadPoolExecutor(max_workers=1)
+        self.priority_thread_pool = QtCore.QThreadPool(self)
+        self.priority_thread_pool.setMaxThreadCount(1)
 
         # Use a separate thread pool for file saving, with a thread count of 1,
         # to avoid race conditions in File._save_and_rename.
-        self.save_thread_pool = ThreadPoolExecutor(max_workers=1)
+        self.save_thread_pool = QtCore.QThreadPool(self)
+        self.save_thread_pool.setMaxThreadCount(1)
 
         if not IS_WIN:
             # Set up signal handling
@@ -231,7 +234,7 @@ class Tagger(QtWidgets.QApplication):
             self.signalfd = socket.socketpair(socket.AF_UNIX, socket.SOCK_STREAM, 0)
 
             self.signalnotifier = QtCore.QSocketNotifier(self.signalfd[1].fileno(),
-                                                         QtCore.QSocketNotifier.Read, self)
+                                                         QtCore.QSocketNotifier.Type.Read, self)
             self.signalnotifier.activated.connect(self.sighandler)
 
             signal.signal(signal.SIGHUP, self.signal)
@@ -246,6 +249,7 @@ class Tagger(QtWidgets.QApplication):
         log.debug("Configuration file path: %r", config.fileName())
 
         log.debug("User directory: %r", os.path.abspath(USER_DIR))
+        log.debug("System long path support: %r", system_supports_long_paths())
 
         # for compatibility with pre-1.3 plugins
         QtCore.QObject.tagger = self
@@ -271,9 +275,6 @@ class Tagger(QtWidgets.QApplication):
         self._acoustid.init()
         self.acoustidmanager = AcoustIDManager(acoustid_api)
 
-        # Setup AcousticBrainz extraction
-        self.ab_extractor = ABExtractor()
-
         self.enable_menu_icons(config.setting['show_menu_icons'])
 
         # Load plugins
@@ -297,12 +298,22 @@ class Tagger(QtWidgets.QApplication):
         self.exit_cleanup = []
         self.stopping = False
 
+        # On macOS temporary files get deleted after 3 days not being accessed.
+        # Touch these files regularly to keep them alive if Picard
+        # is left running for a long time.
+        if IS_MACOS:
+            periodictouch.enable_timer()
+
         # Load release version information
         if self.autoupdate_enabled:
             self.updatecheckmanager = UpdateCheckManager(parent=self.window)
 
+    @property
+    def is_wayland(self):
+        return self.platformName() == 'wayland'
+
     def enable_menu_icons(self, enabled):
-        self.setAttribute(QtCore.Qt.AA_DontShowIconsInMenus, not enabled)
+        self.setAttribute(QtCore.Qt.ApplicationAttribute.AA_DontShowIconsInMenus, not enabled)
 
     def register_cleanup(self, func):
         self.exit_cleanup.append(func)
@@ -319,11 +330,11 @@ class Tagger(QtWidgets.QApplication):
         if not parent:
             parent = self.window
         dialog = QtWidgets.QInputDialog(parent)
-        dialog.setWindowModality(QtCore.Qt.WindowModal)
+        dialog.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
         dialog.setWindowTitle(_("MusicBrainz Account"))
         dialog.setLabelText(_("Authorization code:"))
         status = dialog.exec_()
-        if status == QtWidgets.QDialog.Accepted:
+        if status == QtWidgets.QDialog.DialogCode.Accepted:
             return dialog.textValue()
         else:
             return None
@@ -395,9 +406,9 @@ class Tagger(QtWidgets.QApplication):
         self.stopping = True
         log.debug("Picard stopping")
         self._acoustid.done()
-        self.thread_pool.shutdown()
-        self.save_thread_pool.shutdown()
-        self.priority_thread_pool.shutdown()
+        self.thread_pool.waitForDone()
+        self.save_thread_pool.waitForDone()
+        self.priority_thread_pool.waitForDone()
         self.browser_integration.stop()
         self.webservice.stop()
         self.run_cleanup()
@@ -427,7 +438,7 @@ class Tagger(QtWidgets.QApplication):
     def event(self, event):
         if isinstance(event, thread.ProxyToMainEvent):
             event.run()
-        elif event.type() == QtCore.QEvent.FileOpen:
+        elif event.type() == QtCore.QEvent.Type.FileOpen:
             file = event.file()
             self.add_paths([file])
             if IS_HAIKU:
@@ -523,22 +534,23 @@ class Tagger(QtWidgets.QApplication):
             log.debug("Aborting move since target is invalid")
             return
         self.window.set_sorting(False)
-        if isinstance(target, Cluster):
-            for file in process_events_iter(files):
-                file.move(target)
-        elif isinstance(target, Track):
-            album = target.album
-            for file in process_events_iter(files):
-                file.move(target)
-                if move_to_multi_tracks:  # Assign next file to following track
-                    target = album.get_next_track(target) or album.unmatched_files
-        elif isinstance(target, File):
-            for file in process_events_iter(files):
-                file.move(target.parent)
-        elif isinstance(target, Album):
-            self.move_files_to_album(files, album=target)
-        elif isinstance(target, ClusterList):
-            self.cluster(files)
+        with self.window.metadata_box.ignore_updates:
+            if isinstance(target, Cluster):
+                for file in process_events_iter(files):
+                    file.move(target)
+            elif isinstance(target, Track):
+                album = target.album
+                for file in process_events_iter(files):
+                    file.move(target)
+                    if move_to_multi_tracks:  # Assign next file to following track
+                        target = album.get_next_track(target) or album.unmatched_files
+            elif isinstance(target, File):
+                for file in process_events_iter(files):
+                    file.move(target.parent)
+            elif isinstance(target, Album):
+                self.move_files_to_album(files, album=target)
+            elif isinstance(target, ClusterList):
+                self.cluster(files)
         self.window.set_sorting(True)
 
     def add_files(self, filenames, target=None):
@@ -591,7 +603,7 @@ class Tagger(QtWidgets.QApplication):
     def _scan_paths_recursive(paths, recursive, ignore_hidden):
         local_paths = list(paths)
         while local_paths:
-            current_path = local_paths.pop(0)
+            current_path = normpath(local_paths.pop(0))
             try:
                 if os.path.isdir(current_path):
                     for entry in os.scandir(current_path):
@@ -648,7 +660,9 @@ class Tagger(QtWidgets.QApplication):
                 dialog.search(text)
                 dialog.exec_()
         else:
-            lookup.search_entity(search['entity'], text, adv, mbid_matched_callback=mbid_matched_callback)
+            lookup.search_entity(search['entity'], text, adv,
+                                 mbid_matched_callback=mbid_matched_callback,
+                                 force_browser=force_browser)
 
     def collection_lookup(self):
         """Lookup the users collections on the MusicBrainz website."""
@@ -823,7 +837,11 @@ class Tagger(QtWidgets.QApplication):
         """Reads CD from the selected drive and tries to lookup the DiscID on MusicBrainz."""
         config = get_config()
         if isinstance(action, QtWidgets.QAction):
-            device = action.data()
+            data = action.data()
+            if data == 'logfile:eac':
+                return self.lookup_discid_from_logfile()
+            else:
+                device = data
         elif config.setting["cd_lookup_device"] != '':
             device = config.setting["cd_lookup_device"].split(",", 1)[0]
         else:
@@ -837,6 +855,34 @@ class Tagger(QtWidgets.QApplication):
             partial(self._lookup_disc, disc),
             traceback=self._debug)
 
+    def lookup_discid_from_logfile(self):
+        file_chooser = QtWidgets.QFileDialog(self.window)
+        file_chooser.setNameFilters([
+            _("EAC / XLD / Whipper log files") + " (*.log)",
+            _("All files") + " (*)",
+        ])
+        if file_chooser.exec_():
+            files = file_chooser.selectedFiles()
+            disc = Disc()
+            self.set_wait_cursor()
+            thread.run_task(
+                partial(self._parse_disc_ripping_log, disc, files[0]),
+                partial(self._lookup_disc, disc),
+                traceback=self._debug)
+
+    def _parse_disc_ripping_log(self, disc, path):
+        try:
+            log.debug('Trying to parse "%s" as EAC / XLD log...', path)
+            toc = eaclog.toc_from_file(path)
+        except Exception:
+            try:
+                log.debug('Trying to parse "%s" as Whipper log...', path)
+                toc = whipperlog.toc_from_file(path)
+            except Exception:
+                log.warning('Failed parsing ripping log "%s"', path, exc_info=True)
+                raise
+        disc.put(toc)
+
     @property
     def use_acoustid(self):
         config = get_config()
@@ -863,37 +909,6 @@ class Tagger(QtWidgets.QApplication):
             file.set_pending()
             self._acoustid.fingerprint(file, partial(finished, file))
 
-    def extract_and_submit_acousticbrainz_features(self, objs):
-        """Extract AcousticBrainz features and submit them."""
-        if not self.ab_extractor.available():
-            return
-
-        for file in iter_files_from_objects(objs):
-            # Skip unmatched files
-            if not file.can_extract():
-                log.warning("AcousticBrainz requires a MusicBrainz Recording ID, but file does not have it: %s" % file.filename)
-            # And process matched ones
-            else:
-                file.set_pending()
-
-                # Check if file was either already processed or sent to the AcousticBrainz server
-                if file.acousticbrainz_features_file:
-                    results = (file.acousticbrainz_features_file, 0, "Writing results")
-                    ab_extractor_callback(self, file, results, False)
-                elif file.acousticbrainz_is_duplicate:
-                    results = (None, 0, "Duplicate")
-                    ab_extractor_callback(self, file, results, False)
-                else:
-                    file.acousticbrainz_error = False
-                    # Launch the acousticbrainz on a separate process
-                    log.debug("Extracting AcousticBrainz features from %s" % file.filename)
-                    ab_feature_extraction(
-                        self,
-                        file.metadata["musicbrainz_recordingid"],
-                        file.filename,
-                        partial(ab_extractor_callback, self, file)
-                    )
-
     # =======================================================================
     #  Metadata-based lookups
     # =======================================================================
@@ -911,12 +926,6 @@ class Tagger(QtWidgets.QApplication):
         """Group files with similar metadata to 'clusters'."""
         log.debug("Clustering %r", objs)
         files = iter_files_from_objects(objs)
-        try:
-            file = next(files)
-        except StopIteration:
-            files = self.unclustered_files.files
-        else:
-            files = itertools.chain([file], files)
         thread.run_task(
             partial(self._do_clustering, list(files)),
             partial(self._clustering_finished, callback))
@@ -962,7 +971,7 @@ class Tagger(QtWidgets.QApplication):
     def set_wait_cursor(self):
         """Sets the waiting cursor."""
         super().setOverrideCursor(
-            QtGui.QCursor(QtCore.Qt.WaitCursor))
+            QtGui.QCursor(QtCore.Qt.CursorShape.WaitCursor))
 
     def restore_cursor(self):
         """Restores the cursor set by ``set_wait_cursor``."""
@@ -974,7 +983,7 @@ class Tagger(QtWidgets.QApplication):
                 obj.load(priority=True, refresh=True)
 
     def bring_tagger_front(self):
-        self.window.setWindowState(self.window.windowState() & ~QtCore.Qt.WindowMinimized | QtCore.Qt.WindowActive)
+        self.window.setWindowState(self.window.windowState() & ~QtCore.Qt.WindowState.WindowMinimized | QtCore.Qt.WindowState.WindowActive)
         self.window.raise_()
         self.window.activateWindow()
 
@@ -1033,8 +1042,8 @@ def process_picard_args():
     parser.add_argument("-V", "--long-version", action='store_true',
                         help="display long version information and exit")
     parser.add_argument('FILE', nargs='*')
-    picard_args, unparsed_args = parser.parse_known_args()
-    return picard_args, unparsed_args
+
+    return parser.parse_known_args()[0]
 
 
 def main(localedir=None, autoupdate=True):
@@ -1044,8 +1053,8 @@ def main(localedir=None, autoupdate=True):
     QtWidgets.QApplication.setDesktopFileName(PICARD_DESKTOP_NAME)
 
     # Allow High DPI Support
-    QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
-    QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
+    QtWidgets.QApplication.setAttribute(QtCore.Qt.ApplicationAttribute.AA_UseHighDpiPixmaps)
+    QtWidgets.QApplication.setAttribute(QtCore.Qt.ApplicationAttribute.AA_EnableHighDpiScaling)
     # HighDpiScaleFactorRoundingPolicy is available since Qt 5.14. This is
     # required to support fractional scaling on Windows properly.
     # It causes issues without scaling on Linux, see https://tickets.metabrainz.org/browse/PICARD-1948
@@ -1058,7 +1067,7 @@ def main(localedir=None, autoupdate=True):
 
     signal.signal(signal.SIGINT, signal.SIG_DFL)
 
-    picard_args, unparsed_args = process_picard_args()
+    picard_args = process_picard_args()
     if picard_args.version:
         return version()
     if picard_args.long_version:
@@ -1071,12 +1080,12 @@ def main(localedir=None, autoupdate=True):
     except ImportError:
         pass
 
-    tagger = Tagger(picard_args, unparsed_args, localedir, autoupdate)
+    tagger = Tagger(picard_args, localedir, autoupdate)
 
     # Initialize Qt default translations
     translator = QtCore.QTranslator()
     locale = QtCore.QLocale()
-    translation_path = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath)
+    translation_path = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.LibraryLocation.TranslationsPath)
     log.debug("Looking for Qt locale %s in %s", locale.name(), translation_path)
     if translator.load(locale, "qtbase_", directory=translation_path):
         tagger.installTranslator(translator)
