@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2006-2007 Lukáš Lalinský
 # Copyright (C) 2009 Nikolai Prokoschenko
-# Copyright (C) 2009, 2019-2021 Philipp Wolfer
+# Copyright (C) 2009, 2019-2022 Philipp Wolfer
 # Copyright (C) 2013, 2015, 2018-2021 Laurent Monin
 # Copyright (C) 2016-2017 Sambhav Kothari
 #
@@ -87,7 +87,7 @@ class OptionsPage(QtWidgets.QWidget):
             config.setting[key] = old_options[key]
 
     def display_error(self, error):
-        dialog = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Warning, error.title, error.info, QtWidgets.QMessageBox.Ok, self)
+        dialog = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Icon.Warning, error.title, error.info, QtWidgets.QMessageBox.StandardButton.Ok, self)
         dialog.exec_()
 
     def init_regex_checker(self, regex_edit, regex_error):
