@@ -2,9 +2,9 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2021 Bob Swift
+# Copyright (C) 2021-2022 Bob Swift
 # Copyright (C) 2021 Laurent Monin
-# Copyright (C) 2021 Philipp Wolfer
+# Copyright (C) 2021-2022 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -50,6 +50,7 @@ from picard.const import (
     PICARD_URLS,
 )
 from picard.file import File
+from picard.metadata import Metadata
 from picard.script import (
     ScriptError,
     ScriptParser,
@@ -67,6 +68,7 @@ from picard.util import (
     unique_numbered_title,
     webbrowser2,
 )
+from picard.util.filenaming import WinPathTooLong
 from picard.util.settingsoverride import SettingsOverride
 
 from picard.ui import (
@@ -153,20 +155,22 @@ class ScriptEditorExamples():
         Returns:
             tuple: Example before and after names for the specified file
         """
+        # Operate on a copy of the file object metadata to avoid multiple changes to file metadata.  See PICARD-2508.
+        c_metadata = Metadata()
+        c_metadata.copy(file.metadata)
         try:
-            if self.settings["enable_tagger_scripts"]:
+            # Only apply scripts if the original file metadata has not been changed.
+            if self.settings["enable_tagger_scripts"] and not c_metadata.diff(file.orig_metadata):
                 for s_pos, s_name, s_enabled, s_text in self.settings["list_of_scripts"]:
                     if s_enabled and s_text:
                         parser = ScriptParser()
-                        parser.eval(s_text, file.metadata)
+                        parser.eval(s_text, c_metadata)
             filename_before = file.filename
-            filename_after = file.make_filename(filename_before, file.metadata, self.settings, self.script_text)
+            filename_after = file.make_filename(filename_before, c_metadata, self.settings, self.script_text)
             if not self.settings["move_files"]:
                 return os.path.basename(filename_before), os.path.basename(filename_after)
             return filename_before, filename_after
-        except ScriptError:
-            return "", ""
-        except TypeError:
+        except (ScriptError, TypeError, WinPathTooLong):
             return "", ""
 
     def update_example_listboxes(self, before_listbox, after_listbox):
@@ -261,7 +265,7 @@ class ScriptEditorExamples():
             'musicbrainz_albumartistid': 'b10bbbfc-cf9e-42e0-be17-e2c3e1d2600d',
             'musicbrainz_artistid': 'b10bbbfc-cf9e-42e0-be17-e2c3e1d2600d',
             'musicbrainz_recordingid': 'ed052ae1-c950-47f2-8d2b-46e1b58ab76c',
-            'musicbrainz_releasetrackid': '392639f5-5629-477e-b04b-93bffa703405',
+            'musicbrainz_trackid': '392639f5-5629-477e-b04b-93bffa703405',
         })
         yield efile
 
@@ -294,7 +298,7 @@ class ScriptEditorExamples():
             '~extension': 'mp3',
             'musicbrainz_albumid': '4b50c71e-0a07-46ac-82e4-cb85dc0c9bdd',
             'musicbrainz_recordingid': 'b3c487cb-0e55-477d-8df3-01ec6590f099',
-            'musicbrainz_releasetrackid': 'f8649a05-da39-39ba-957c-7abf8f9012be',
+            'musicbrainz_trackid': 'f8649a05-da39-39ba-957c-7abf8f9012be',
             'musicbrainz_albumartistid': '89ad4ac3-39f7-470e-963a-56509c546377',
             'musicbrainz_artistid': ['7b593455-d207-482c-8c6f-19ce22c94679', '9e082466-2390-40d1-891e-4803531f43fd'],
         })
@@ -312,13 +316,13 @@ def confirmation_dialog(parent, message):
         bool: True if accepted, otherwise False.
     """
     dialog = QtWidgets.QMessageBox(
-        QtWidgets.QMessageBox.Warning,
+        QtWidgets.QMessageBox.Icon.Warning,
         _('Confirm'),
         message,
-        QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel,
+        QtWidgets.QMessageBox.StandardButton.Ok | QtWidgets.QMessageBox.StandardButton.Cancel,
         parent
     )
-    return dialog.exec_() == QtWidgets.QMessageBox.Ok
+    return dialog.exec_() == QtWidgets.QMessageBox.StandardButton.Ok
 
 
 def synchronize_vertical_scrollbars(widgets):
@@ -329,8 +333,8 @@ def synchronize_vertical_scrollbars(widgets):
     """
     # Set highlight colors for selected list items
     example_style = widgets[0].palette()
-    highlight_bg = example_style.color(QPalette.Active, QPalette.Highlight)
-    highlight_fg = example_style.color(QPalette.Active, QPalette.HighlightedText)
+    highlight_bg = example_style.color(QPalette.ColorGroup.Active, QPalette.ColorRole.Highlight)
+    highlight_fg = example_style.color(QPalette.ColorGroup.Active, QPalette.ColorRole.HighlightedText)
     stylesheet = "QListView::item:selected { color: " + highlight_fg.name() + "; background-color: " + highlight_bg.name() + "; }"
 
     def _sync_scrollbar_vert(widget, value):
@@ -399,9 +403,9 @@ def populate_script_selection_combo_box(naming_scripts, selected_script_id, comb
 class NotEmptyValidator(QtGui.QValidator):
     def validate(self, text: str, pos):
         if bool(text.strip()):
-            state = QtGui.QValidator.Acceptable
+            state = QtGui.QValidator.State.Acceptable
         else:
-            state = QtGui.QValidator.Intermediate  # so that field can be made empty temporarily
+            state = QtGui.QValidator.State.Intermediate  # so that field can be made empty temporarily
         return state, text, pos
 
 
@@ -429,7 +433,7 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
     signal_selection_changed = QtCore.pyqtSignal()
     signal_index_changed = QtCore.pyqtSignal()
 
-    default_script_directory = os.path.normpath(QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.DocumentsLocation))
+    default_script_directory = os.path.normpath(QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.StandardLocation.DocumentsLocation))
     default_script_filename = "picard_naming_script.ptsp"
 
     Profile = namedtuple('Profile', ['id', 'title', 'script_id'])
@@ -477,22 +481,22 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
         self.reset_button = QtWidgets.QPushButton(_('Reset'))
         self.reset_button.setToolTip(self.reset_action.toolTip())
         self.reset_button.clicked.connect(self.reload_from_config)
-        self.ui.buttonbox.addButton(self.reset_button, QtWidgets.QDialogButtonBox.ActionRole)
+        self.ui.buttonbox.addButton(self.reset_button, QtWidgets.QDialogButtonBox.ButtonRole.ActionRole)
 
-        self.save_button = self.ui.buttonbox.addButton(_("Make It So!"), QtWidgets.QDialogButtonBox.AcceptRole)
+        self.save_button = self.ui.buttonbox.addButton(_("Make It So!"), QtWidgets.QDialogButtonBox.ButtonRole.AcceptRole)
         self.save_button.setToolTip(self.save_action.toolTip())
         self.ui.buttonbox.accepted.connect(self.make_it_so)
 
-        self.close_button = self.ui.buttonbox.addButton(QtWidgets.QDialogButtonBox.Cancel)
+        self.close_button = self.ui.buttonbox.addButton(QtWidgets.QDialogButtonBox.StandardButton.Cancel)
         self.close_button.setToolTip(self.close_action.toolTip())
         self.ui.buttonbox.rejected.connect(self.close)
 
-        self.ui.buttonbox.addButton(QtWidgets.QDialogButtonBox.Help)
+        self.ui.buttonbox.addButton(QtWidgets.QDialogButtonBox.StandardButton.Help)
         self.ui.buttonbox.helpRequested.connect(self.show_help)
 
         # Add links to edit script metadata
         self.ui.script_title.installEventFilter(self)
-        self.ui.script_title.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
+        self.ui.script_title.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.ActionsContextMenu)
         self.ui.script_title.addAction(self.details_action)
 
         self.ui.file_naming_format.setEnabled(True)
@@ -535,7 +539,7 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
             parent (object): Parent to set for the instance
         """
         if self.parent() != parent:
-            flags = self.windowFlags() | QtCore.Qt.Window
+            flags = self.windowFlags() | QtCore.Qt.WindowType.Window
             super().setParent(parent, flags)
             # Set appropriate state of script selector in parent
             save_enabled = self.save_button.isEnabled()
@@ -634,7 +638,7 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
         help_menu.setToolTipsVisible(True)
 
         self.help_action = QtWidgets.QAction(_("&Help..."), self)
-        self.help_action.setShortcut(QtGui.QKeySequence.HelpContents)
+        self.help_action.setShortcut(QtGui.QKeySequence.StandardKey.HelpContents)
         self.help_action.triggered.connect(self.show_help)
         help_menu.addAction(self.help_action)
 
@@ -704,9 +708,9 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
         """Process selected events.
         """
         evtype = event.type()
-        if evtype in {QtCore.QEvent.WindowActivate, QtCore.QEvent.FocusIn}:
+        if evtype in {QtCore.QEvent.Type.WindowActivate, QtCore.QEvent.Type.FocusIn}:
             self.update_examples()
-        elif object == self.ui.script_title and evtype == QtCore.QEvent.MouseButtonDblClick:
+        elif object == self.ui.script_title and evtype == QtCore.QEvent.Type.MouseButtonDblClick:
             self.details_action.trigger()
             return True
         return False
@@ -1095,14 +1099,14 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
         profile = self.is_used_in_profile()
         if profile is not None:
             QtWidgets.QMessageBox(
-                QtWidgets.QMessageBox.Warning,
+                QtWidgets.QMessageBox.Icon.Warning,
                 _("Error Deleting Script"),
                 _(
                     "The script could not be deleted because it is used in one of the user profiles."
                     "\n\n"
                     "Profile: %s"
                 ) % profile.title,
-                QtWidgets.QMessageBox.Ok,
+                QtWidgets.QMessageBox.StandardButton.Ok,
                 self
             ).exec_()
             return
@@ -1209,7 +1213,7 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
                 if title != existing_script['title']:
                     continue
                 box = QtWidgets.QMessageBox()
-                box.setIcon(QtWidgets.QMessageBox.Question)
+                box.setIcon(QtWidgets.QMessageBox.Icon.Question)
                 box.setWindowTitle(_('Confirm'))
                 box.setText(
                     _(
@@ -1217,12 +1221,12 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
                         "\n"
                         "Do you want to overwrite it, add as a copy or cancel?"
                     ).format(script_name=title,))
-                box.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No | QtWidgets.QMessageBox.Cancel)
-                buttonY = box.button(QtWidgets.QMessageBox.Yes)
+                box.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Yes | QtWidgets.QMessageBox.StandardButton.No | QtWidgets.QMessageBox.StandardButton.Cancel)
+                buttonY = box.button(QtWidgets.QMessageBox.StandardButton.Yes)
                 buttonY.setText(_("Overwrite"))
-                buttonN = box.button(QtWidgets.QMessageBox.No)
+                buttonN = box.button(QtWidgets.QMessageBox.StandardButton.No)
                 buttonN.setText(_("Copy"))
-                box.setDefaultButton(QtWidgets.QMessageBox.Cancel)
+                box.setDefaultButton(QtWidgets.QMessageBox.StandardButton.Cancel)
                 box.exec_()
 
                 if box.clickedButton() == buttonY:
@@ -1278,8 +1282,8 @@ class ScriptEditorDialog(PicardDialog, SingletonDialog):
         # Ignore scripting errors, those are handled inline
         if not isinstance(error, ScriptCheckError):
             dialog = QtWidgets.QMessageBox(
-                QtWidgets.QMessageBox.Warning, error.title,
-                error.info, QtWidgets.QMessageBox.Ok, self
+                QtWidgets.QMessageBox.Icon.Warning, error.title,
+                error.info, QtWidgets.QMessageBox.StandardButton.Ok, self
             )
             dialog.exec_()
 
@@ -1339,9 +1343,9 @@ class ScriptDetailsEditor(PicardDialog):
         self.ui.script_license.setReadOnly(self.readonly)
         self.ui.script_description.setReadOnly(self.readonly)
 
-        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.Close).setVisible(self.readonly)
-        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.Cancel).setVisible(not self.readonly)
-        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.Save).setVisible(not self.readonly)
+        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.StandardButton.Close).setVisible(self.readonly)
+        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.StandardButton.Cancel).setVisible(not self.readonly)
+        self.ui.buttonBox.button(QtWidgets.QDialogButtonBox.StandardButton.Save).setVisible(not self.readonly)
         self.ui.last_updated_now.setVisible(not self.readonly)
         self.ui.buttonBox.setFocus()
 
@@ -1388,10 +1392,10 @@ class ScriptDetailsEditor(PicardDialog):
         title = self.ui.script_title.text().strip()
         if not title:
             QtWidgets.QMessageBox(
-                QtWidgets.QMessageBox.Critical,
+                QtWidgets.QMessageBox.Icon.Critical,
                 _("Error"),
                 _("The script title must not be empty."),
-                QtWidgets.QMessageBox.Ok,
+                QtWidgets.QMessageBox.StandardButton.Ok,
                 self
             ).exec_()
             return
