@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2004 Robert Kaye
 # Copyright (C) 2006-2009, 2011-2012, 2014 Lukáš Lalinský
-# Copyright (C) 2008-2011, 2014, 2018-2021 Philipp Wolfer
+# Copyright (C) 2008-2011, 2014, 2018-2022 Philipp Wolfer
 # Copyright (C) 2009 Carlin Mangar
 # Copyright (C) 2009 david
 # Copyright (C) 2010 fatih
@@ -42,9 +42,9 @@
 import builtins
 from collections import namedtuple
 from collections.abc import Mapping
-import html
 from itertools import chain
 import json
+from locale import strxfrm as _strxfrm
 import ntpath
 from operator import attrgetter
 import os
@@ -73,6 +73,21 @@ from picard.const.sys import (
 )
 
 
+if IS_WIN:
+    import winreg
+
+# Windows path length constraints
+# See https://docs.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation
+# the entire path's length (260 - 1 null character)
+WIN_MAX_FILEPATH_LEN = 259
+# the entire parent directory path's length must leave room for a 8.3 filename
+WIN_MAX_DIRPATH_LEN = WIN_MAX_FILEPATH_LEN - 12
+# a single node's (directory or file) length
+WIN_MAX_NODE_LEN = 255
+# Prefix for long paths in Windows API
+WIN_LONGPATH_PREFIX = '\\\\?\\'
+
+
 class LockableObject(QtCore.QObject):
 
     """Read/write lockable object."""
@@ -165,13 +180,63 @@ def decode_filename(filename):
         return filename.decode(_io_encoding)
 
 
+def _check_windows_min_version(major, build):
+    try:
+        v = sys.getwindowsversion()
+        return v.major >= major and v.build >= build
+    except AttributeError:
+        return False
+
+
+def system_supports_long_paths():
+    """Detects long path support.
+
+    On Windows returns True, only if long path support is enabled in the registry (Windows 10 1607 or later).
+    All other systems return always True.
+    """
+    if not IS_WIN:
+        return True
+    try:
+        # Use cached value
+        return system_supports_long_paths._supported
+    except AttributeError:
+        pass
+    try:
+        # Long path support can be enabled in Windows 10 version 1607 or later
+        if _check_windows_min_version(10, 14393):
+            with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,
+                                r"SYSTEM\CurrentControlSet\Control\FileSystem") as key:
+                supported = winreg.QueryValueEx(key, "LongPathsEnabled")[0] == 1
+        else:
+            supported = False
+        system_supports_long_paths._supported = supported
+        return supported
+    except OSError:
+        log.info('Failed reading LongPathsEnabled from registry')
+        return False
+
+
 def normpath(path):
+    path = os.path.normpath(path)
     try:
         path = os.path.realpath(path)
     except OSError as why:
         # realpath can fail if path does not exist or is not accessible
+        # or on Windows if drives are mounted without mount manager
+        # (see https://tickets.metabrainz.org/browse/PICARD-2425).
         log.warning('Failed getting realpath for "%s": %s', path, why)
-    return os.path.normpath(path)
+    # If the path is longer than 259 characters on Windows, prepend the \\?\
+    # prefix. This enables access to long paths using the Windows API. See
+    # https://docs.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation
+    if IS_WIN and not system_supports_long_paths():
+        path = win_prefix_longpath(path)
+    return path
+
+
+def win_prefix_longpath(path):
+    if len(path) > WIN_MAX_FILEPATH_LEN and not path.startswith(WIN_LONGPATH_PREFIX):
+        path = WIN_LONGPATH_PREFIX + path
+    return path
 
 
 def is_absolute_path(path):
@@ -415,6 +480,38 @@ def throttle(interval):
     return decorator
 
 
+class IgnoreUpdatesContext:
+    """Context manager for holding a boolean value, indicating whether updates are performed or not.
+    By default the context resolves to False. If entered it is True. This allows
+    to temporarily set a state on a block of code like:
+
+        ignore_changes = IgnoreUpdatesContext()
+        # Initially ignore_changes is False
+        with ignore_changes:
+            # Perform some tasks with ignore_changes now being True
+            ...
+        # ignore_changes is False again
+
+    The code actually doing updates can check `ignore_changes` and only perform
+    updates if it is `False`.
+    """
+
+    def __init__(self, onexit=None):
+        self._entered = 0
+        self._onexit = onexit
+
+    def __enter__(self):
+        self._entered += 1
+
+    def __exit__(self, type, value, tb):
+        self._entered -= 1
+        if self._onexit:
+            self._onexit()
+
+    def __bool__(self):
+        return self._entered > 0
+
+
 def uniqify(seq):
     """Uniqify a list, preserving order"""
     return list(iter_unique(seq))
@@ -431,13 +528,13 @@ _tracknum_regexps = [re.compile(r, re.I) for r in (
     # search for explicit track number (prefix "track")
     r"track[\s_-]*(?:(?:no|nr)\.?)?[\s_-]*(?P<number>\d+)",
     # search for 1- or 2-digit number at start of string (additional leading zeroes are allowed)
-    # An optional disc number preceeding the track number is ignored.
+    # An optional disc number preceding the track number is ignored.
     r"^(?:\d+[\s_-])?(?P<number>0*\d{1,2})(?:\.)[^0-9,]",  # "99. ", but not "99.02"
     r"^(?:\d+[\s_-])?(?P<number>0*\d{1,2})[^0-9,.s]",
     # search for 2-digit number at end of string (additional leading zeroes are allowed)
-    r"[^0-9,.](?P<number>0*\d{2})$",
-    r"[^0-9,.]\[(?P<number>0*\d{1,2})\]$",
-    r"[^0-9,.]\((?P<number>0*\d{2})\)$",
+    r"[^0-9,.\w](?P<number>0*\d{2})$",
+    r"[^0-9,.\w]\[(?P<number>0*\d{1,2})\]$",
+    r"[^0-9,.\w]\((?P<number>0*\d{2})\)$",
     # File names which consist of only a number
     r"^(?P<number>\d+)$",
 )]
@@ -580,13 +677,14 @@ def build_qurl(host, port=80, path=None, queryargs=None):
     """
     url = QtCore.QUrl()
     url.setHost(host)
-    url.setPort(port)
 
-    if host in MUSICBRAINZ_SERVERS or port == 443:
+    if port == 443 or host in MUSICBRAINZ_SERVERS:
         url.setScheme("https")
-        url.setPort(443)
+    elif port == 80:
+        url.setScheme("http")
     else:
         url.setScheme("http")
+        url.setPort(port)
 
     if path is not None:
         url.setPath(path)
@@ -654,10 +752,6 @@ def convert_to_string(obj):
 builtins.__dict__['string_'] = convert_to_string
 
 
-def htmlescape(string):
-    return html.escape(string, quote=False)
-
-
 def load_json(data):
     """Deserializes a string or bytes like json response and converts
     it to a python object.
@@ -723,15 +817,29 @@ BestMatch = namedtuple('BestMatch', ('similarity', 'result'))
 
 
 def sort_by_similarity(candidates):
+    """Sorts the objects in candidates by similarity.
+
+    Args:
+        candidates: Iterable with objects having a `similarity`  attribute
+    Returns: List of candidates sorted by similarity (highest similarity first)
+    """
     return sorted(
-        candidates(),
+        candidates,
         reverse=True,
         key=attrgetter('similarity')
     )
 
 
 def find_best_match(candidates, no_match):
-    best_match = max(candidates(), key=attrgetter('similarity'), default=no_match)
+    """Returns a BestMatch based on the similarity of candidates.
+
+    Args:
+        candidates: Iterable with objects having a `similarity`  attribute
+        no_match: Match to return if there was no candidate
+
+    Returns: `BestMatch` with the similarity and the matched object as result.
+    """
+    best_match = max(candidates, key=attrgetter('similarity'), default=no_match)
     return BestMatch(similarity=best_match.similarity, result=best_match)
 
 
@@ -967,3 +1075,37 @@ def get_base_title(title):
     """
     suffix = _(DEFAULT_COPY_TEXT)
     return get_base_title_with_suffix(title, suffix)
+
+
+def iter_exception_chain(err):
+    """Iterate over the exception chain.
+    Yields this exception and all __context__ and __cause__ exceptions"""
+    yield err
+    if hasattr(err, '__context__'):
+        yield from iter_exception_chain(err.__context__)
+    if hasattr(err, '__cause__'):
+        yield from iter_exception_chain(err.__cause__)
+
+
+def any_exception_isinstance(error, type_):
+    """Returns True, if any exception in the exception chain is instance of type_."""
+    return any(isinstance(err, type_) for err in iter_exception_chain(error))
+
+
+def strxfrm(string):
+    """Transforms a string to one that can be used in locale-aware comparisons.
+
+    Wrapper around locale.strxfrm, that never throws OSError or ValueError. If an
+    error occurs this function will return the string converted to lower case.
+    Also null bytes in the input string will be ignored.
+
+    Args:
+        string: The string to convert
+
+    Returns: A string that can be compared locale-aware
+    """
+    try:
+        return _strxfrm(string.replace('\0', ''))
+    except (OSError, ValueError) as err:
+        log.warning('strxfrm(%r) failed: %r', string, err)
+        return string.lower()
