@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2017 Sambhav Kothari
 # Copyright (C) 2018, 2020-2021 Laurent Monin
-# Copyright (C) 2018-2021 Philipp Wolfer
+# Copyright (C) 2018-2022 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -47,6 +47,11 @@ def escape_lucene_query(text):
     return re.sub(r'([+\-&|!(){}\[\]\^"~*?:\\/])', r'\\\1', text)
 
 
+def build_lucene_query(args):
+    return ' '.join('%s:(%s)' % (item, escape_lucene_query(value))
+                    for item, value in args.items() if value)
+
+
 def _wrap_xml_metadata(data):
     return ('<?xml version="1.0" encoding="UTF-8"?>'
             '<metadata xmlns="http://musicbrainz.org/ns/mmd-2.0#">%s</metadata>'
@@ -162,12 +167,7 @@ class MBAPIHelper(APIHelper):
                 query = escape_lucene_query(kwargs["query"]).strip().lower()
                 filters.append(("dismax", 'true'))
         else:
-            query = []
-            for name, value in kwargs.items():
-                value = escape_lucene_query(value).strip().lower()
-                if value:
-                    query.append('%s:(%s)' % (name, value))
-            query = ' '.join(query)
+            query = build_lucene_query(kwargs)
 
         if query:
             filters.append(("query", query))
@@ -191,18 +191,22 @@ class MBAPIHelper(APIHelper):
     def find_artists(self, handler, **kwargs):
         return self._find('artist', handler, **kwargs)
 
-    def _browse(self, entitytype, handler, inc=None, **kwargs):
+    def _browse(self, entitytype, handler, inc=None, queryargs=None, mblogin=False):
         path_list = (entitytype, )
-        queryargs = kwargs
+        if queryargs is None:
+            queryargs = {}
         if inc:
             queryargs["inc"] = "+".join(inc)
         return self.get(path_list, handler, queryargs=queryargs,
-                        priority=True, important=True, mblogin=False,
+                        priority=True, important=True, mblogin=mblogin,
                         refresh=False)
 
     def browse_releases(self, handler, **kwargs):
         inc = ("media", "labels")
-        return self._browse("release", handler, inc, **kwargs)
+        return self._browse("release", handler, inc, queryargs=kwargs)
+
+    def browse_recordings(self, handler, inc, **kwargs):
+        return self._browse('recording', handler, inc, queryargs=kwargs)
 
     @staticmethod
     def _xml_ratings(ratings):
@@ -294,11 +298,9 @@ class AcoustIdAPIHelper(APIHelper):
         config = get_config()
         args = {'user': config.setting["acoustid_apikey"]}
         for i, submission in enumerate(submissions):
-            args['fingerprint.%d' % i] = submission.fingerprint
-            args['duration.%d' % i] = str(submission.duration)
-            args['mbid.%d' % i] = submission.recordingid
-            if submission.puid:
-                args['puid.%d' % i] = submission.puid
+            for key, value in submission.args.items():
+                if value:
+                    args[".".join((key, str(i)))] = value
         return args
 
     def submit_acoustid_fingerprints(self, submissions, handler):
