@@ -2,7 +2,7 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2019-2021 Philipp Wolfer
+# Copyright (C) 2019-2022 Philipp Wolfer
 # Copyright (C) 2020 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
@@ -21,12 +21,14 @@
 
 
 import base64
-import logging
 import os
+from unittest.mock import patch
 
 from mutagen.flac import (
     Padding,
     Picture,
+    SeekPoint,
+    SeekTable,
     VCFLACDict,
 )
 
@@ -35,10 +37,7 @@ from test.picardtestcase import (
     create_fake_png,
 )
 
-from picard import (
-    config,
-    log,
-)
+from picard import config
 from picard.coverart.image import CoverArtImage
 from picard.formats import vorbis
 from picard.formats.util import open_ as open_format
@@ -82,10 +81,7 @@ class CommonVorbisTests:
     class VorbisTestCase(CommonTests.TagFormatsTestCase):
         def test_invalid_rating(self):
             filename = os.path.join('test', 'data', 'test-invalid-rating.ogg')
-            old_log_level = log.get_effective_level()
-            log.set_level(logging.ERROR)
             metadata = load_metadata(filename)
-            log.set_level(old_log_level)
             self.assertEqual(metadata["~rating"], "THERATING")
 
         def test_supports_tags(self):
@@ -250,6 +246,43 @@ class FLACTest(CommonVorbisTests.VorbisTestCase):
                 self.assertGreater(f.metadata_blocks.index(b), tagindex)
         self.assertTrue(haspics, "Picture block expected, none found")
 
+    @patch.object(vorbis, 'flac_remove_empty_seektable')
+    def test_setting_fix_missing_seekpoints_flac(self, mock_flac_remove_empty_seektable):
+        save_metadata(self.filename, Metadata())
+        mock_flac_remove_empty_seektable.assert_not_called()
+        self.set_config_values({
+            'fix_missing_seekpoints_flac': True
+        })
+        save_metadata(self.filename, Metadata())
+        mock_flac_remove_empty_seektable.assert_called_once()
+
+    @skipUnlessTestfile
+    def test_flac_remove_empty_seektable_remove_empty(self):
+        f = load_raw(self.filename)
+        # Add an empty seek table
+        seektable = SeekTable(None)
+        f.seektable = seektable
+        f.metadata_blocks.append(seektable)
+        # This is a zero length file. The empty seektable should get removed
+        vorbis.flac_remove_empty_seektable(f)
+        self.assertIsNone(f.seektable)
+        self.assertNotIn(seektable, f.metadata_blocks)
+
+    @skipUnlessTestfile
+    def test_flac_remove_empty_seektable_keep_existing(self):
+        f = load_raw(self.filename)
+        # Add an non-empty seek table
+        seektable = SeekTable(None)
+        seekpoint = SeekPoint(0, 0, 0)
+        seektable.seekpoints.append(seekpoint)
+        f.seektable = seektable
+        f.metadata_blocks.append(seektable)
+        # Existing non-empty seektable should be kept
+        vorbis.flac_remove_empty_seektable(f)
+        self.assertEqual(seektable, f.seektable)
+        self.assertIn(seektable, f.metadata_blocks)
+        self.assertEqual([seekpoint], f.seektable.seekpoints)
+
 
 class OggVorbisTest(CommonVorbisTests.VorbisTestCase):
     testfile = 'test.ogg'
