@@ -4,8 +4,8 @@
 #
 # Copyright (C) 2017 Sophist-UK
 # Copyright (C) 2018, 2020 Wieland Hoffmann
-# Copyright (C) 2018-2020 Laurent Monin
-# Copyright (C) 2018-2020 Philipp Wolfer
+# Copyright (C) 2018-2021 Laurent Monin
+# Copyright (C) 2018-2021 Philipp Wolfer
 # Copyright (C) 2020 dukeyin
 #
 # This program is free software; you can redistribute it and/or
@@ -41,6 +41,7 @@ from picard.metadata import (
     MULTI_VALUED_JOINER,
     Metadata,
     MultiMetadataProxy,
+    trackcount_score,
     weights_from_preferred_countries,
     weights_from_preferred_formats,
     weights_from_release_type_scores,
@@ -570,6 +571,41 @@ class CommonTests:
                 match = metadata.compare_to_release(release, Cluster.comparison_weights)
                 self.assertEqual(sim, match.similarity)
 
+        def test_compare_to_release_parts_totaltracks(self):
+            release = load_test_json('release_multidisc.json')
+            metadata = Metadata()
+            weights = {"totaltracks": 30}
+            release_to_metadata(release, metadata)
+            for totaltracks, sim in ((4, 1.0), (3, 1.0), (2, 0.3), (5, 0.0)):
+                metadata['totaltracks'] = totaltracks
+                parts = metadata.compare_to_release_parts(release, weights)
+                self.assertIn((sim, 30), parts)
+
+        def test_compare_to_release_parts_totalalbumtracks(self):
+            release = load_test_json('release_multidisc.json')
+            metadata = Metadata()
+            weights = {"totalalbumtracks": 30}
+            release_to_metadata(release, metadata)
+            for totaltracks, sim in ((7, 1.0), (6, 0.3), (8, 0.0)):
+                metadata['~totalalbumtracks'] = totaltracks
+                parts = metadata.compare_to_release_parts(release, weights)
+                self.assertIn((sim, 30), parts)
+
+        def test_compare_to_release_parts_totalalbumtracks_totaltracks_fallback(self):
+            release = load_test_json('release_multidisc.json')
+            metadata = Metadata()
+            weights = {"totalalbumtracks": 30}
+            release_to_metadata(release, metadata)
+            for totaltracks, sim in ((7, 1.0), (6, 0.3), (8, 0.0)):
+                metadata['totaltracks'] = totaltracks
+                parts = metadata.compare_to_release_parts(release, weights)
+                self.assertIn((sim, 30), parts)
+
+        def test_trackcount_score(self):
+            self.assertEqual(1.0, trackcount_score(5, 5))
+            self.assertEqual(0.0, trackcount_score(6, 5))
+            self.assertEqual(0.3, trackcount_score(4, 5))
+
         def test_weights_from_release_type_scores(self):
             release = load_test_json('release.json')
             parts = []
